// radar.c: Radar related functions for Bapple2.  Created on April 18th, 1998

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>

#include <allegro.h>
#include "bapple2.h"
#include "bap2dat.h"
#include "globals.h"
#include "map.h"
#include "radar.h"

void init_radar(void);
void process_radar(void);
radar_t *create_radar(char type, int interval, int speed, int range, int x, int y, unit_t *u);
void update_point(BITMAP *bmp, int x, int y, int d);

/* init_radar(): ----------------------------------------------------------
 *   Initializes the radar system, creating radars for the player's main
 *   base and mini-bases.
 */
void init_radar(void)
{
	struct BASE_NODE *node;
	static char first_init = 1;

	if (!first_init)
		return;
	else
		first_init = 0;

	radar = NULL;

	// Create a radar for the player's main base
	if (player.mbase)
		create_radar(RADAR_NORMAL, 4000, 80, 24, player.mbase->x + 2, player.mbase->y + 2, NULL);

	// Create radars for the player's mini-bases
	for (node = base; node; node = node->next)
		if (node->owner == player.id)
			create_radar(RADAR_NORMAL, 4000, 100, 14, node->x + 1, node->y + 1, NULL);
}

/* create_radar(): --------------------------------------------------------
 *   Creates a radar with the given properties.
 */
radar_t *create_radar(char type, int interval, int speed, int range, int x, int y, unit_t *u)
{
	radar_t *r;

	// Allocate memory for new radar and add it to the list
	r = (radar_t *) malloc(sizeof(radar_t));
	r->next = radar;
	radar = r;

	// Assign explicit creation parameters
	r->type = type;
	r->interval = interval;
	r->speed = speed;
	r->range = range;
	r->x = x;
	r->y = y;
	r->u = u;

	// Assign implicit creation parameters
	r->last_sweep_time = ms_time;
	r->active = 0;

	return r;
}

/* process_radar(): -------------------------------------------------------
 *   Processes all radars in the world - makes them expand and uncover
 *   hidden terrain and objects.
 */
void process_radar(void)
{
	radar_t *r;
	static char first_process = 1;
	int x, y;

	// Automatically initialize the radar system on the first processing call
	if (first_process)
	{
		first_process = 0;
		init_radar();
	}

	// Process radar
	for (r = radar; r; r = r->next)
	{
		// If we're not processing a radar sweep, see if we should be
		if ((!r->active) && (ms_time - r->last_sweep_time > r->interval))
		{
			r->active = 1;
			r->last_sweep_tick = r->last_sweep_time = ms_time;
			r->radius = 0;
		}
		// If we're processing this radar, see if we should advance to the next tick
		if (r->active)
		{
			// Increment the radius and process the sweep
			while (ms_time - r->last_sweep_tick > r->speed)
			{
				// Increment the radius
				r->radius++;
				r->last_sweep_tick += r->speed;
				if (r->radius >= r->range - 2)
					break;
				// Update visible map area (terrain_bmp + mapobj_bmp)
				if (r->u)
					do_circle(NULL, r->u->x, r->u->y, r->radius, 0, update_point);
				else
					do_circle(NULL, r->x, r->y, r->radius, 0, update_point);
			}
			// If the radius is large enough, stop the sweep
			if (r->radius >= r->range)
				r->active = 0;
		}
	}
}

/* update_point(): --------------------------------------------------------
 *   Updates a point on the visible mini-map by reading pixels from the
 *   mapobj_bmp (map objects bitmap, which contains stuff like units) and
 *   the terrain_bmp (terrain bitmap, the background) and writing to the
 *   vismap_bmp (the actual visible bitmap that is the mini-map).
 */
void update_point(BITMAP *bmp, int x, int y, int d)
{
	int c;

	// Get a pixel from the object map
	c = getpixel(mapobj_bmp, x, y);
	// If a negative # was returned, we're outside the bitmap
	if (c < 0)
		return;
	// If a 0 was returned, there is nothing at this tile so get terrain instead
	if (!c)
		c = getpixel(terrain_bmp, x, y);
	// Write our pixel value to the visible map
	putpixel(vismap_bmp, x, y, c);
	putpixel(vismap_bmp, x+1, y, c);
}
